"""
Django settings for src project.

Generated by 'django-admin startproject' using Django 5.2.6.

For more information on this file, see
https://docs.djangoproject.com/en/5.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/5.2/ref/settings/
"""

import os
import sys
from pathlib import Path

from authorizenet.constants import constants

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

ALLOWED_HOSTS = ["127.0.0.1", "localhost"]
DEBUG = True
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"
DJANGO_ENCRYPTED_FIELD_ALGORITHM = os.getenv(
    "DJANGO_ENCRYPTED_FIELD_ALGORITHM", "SS20"
)
DJANGO_ENCRYPTED_FIELD_KEY = os.getenv(
    "DJANGO_ENCRYPTED_FIELD_KEY",
    b"\x8dS\xf7'xT\\\xc9O\x91\x8f\xfc\xd5\xc9\xaf\\.\xb5=J\xc9\x10\xd4\x1b9Zl\x19\xdd\xcc+\xcf",
)
LANGUAGE_CODE = "en-us"
MERCHANT_AUTH_ENVIRONMENT = constants.SANDBOX
MERCHANT_AUTH_LOGIN_ID = os.getenv("MERCHANT_AUTH_LOGIN_ID")
MERCHANT_AUTH_TRANSACTION_KEY = os.getenv("MERCHANT_AUTH_TRANSACTION_KEY")
MERCHANT_AUTH_VALIDATION_MODE = "liveMode"
ROOT_URLCONF = "src.urls"
SECRET_KEY = (
    "django-insecure-k_hy$%724=-h11l@^pur0$#crj0u!#w&68=i$)k9!jxb)yia0k"
)
STATIC_URL = "static/"
TIME_ZONE = "US/Central"
USE_I18N = False
USE_TZ = True
WSGI_APPLICATION = "src.wsgi.application"

TASKS = {
    "default": {"BACKEND": "django.tasks.backends.immediate.ImmediateBackend"}
}

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.forms",
    "terminusgps_payments.apps.TerminusgpsPaymentsConfig",
]

LOGGING = {
    "version": 1,
    "disable_existing_loggers": "test" in sys.argv,
    "handlers": {"console": {"class": "logging.StreamHandler"}},
    "root": {"handlers": ["console"], "level": "WARNING"},
    "loggers": {
        "django": {
            "handlers": ["console"],
            "level": os.getenv("DJANGO_LOG_LEVEL", "INFO"),
            "propagate": False,
        },
        "terminusgps_payments": {
            "handlers": ["console"],
            "level": os.getenv("TERMINUSGPS_PAYMENTS_LOG_LEVEL", "INFO"),
            "propagate": True,
        },
    },
}

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]


TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
    }
}

CACHES = {
    "default": {"BACKEND": "django.core.cache.backends.dummy.DummyCache"}
}

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"
    },
    {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"
    },
]



from django.urls import path

from . import views

app_name = "terminusgps_payments"
urlpatterns = [
    path(
        "subscriptions/create/",
        views.SubscriptionCreateView.as_view(),
        name="create subscriptions",
    ),
    path(
        "subscriptions/<int:pk>/detail/",
        views.SubscriptionDetailView.as_view(),
        name="detail subscriptions",
    ),
    path(
        "subscriptions/<int:pk>/update/",
        views.SubscriptionUpdateView.as_view(),
        name="update subscriptions",
    ),
    path(
        "subscriptions/<int:pk>/delete/",
        views.SubscriptionDeleteView.as_view(),
        name="delete subscriptions",
    ),
    path(
        "address-profiles/create/",
        views.CustomerAddressProfileCreateView.as_view(),
        name="create address profiles",
    ),
    path(
        "address-profiles/list/",
        views.CustomerAddressProfileListView.as_view(),
        name="list address profiles",
    ),
    path(
        "address-profiles/sync/",
        views.CustomerAddressProfileSyncView.as_view(),
        name="sync address profiles",
    ),
    path(
        "address-profiles/<int:pk>/detail/",
        views.CustomerAddressProfileDetailView.as_view(),
        name="detail address profiles",
    ),
    path(
        "address-profiles/<int:pk>/delete/",
        views.CustomerAddressProfileDeleteView.as_view(),
        name="delete address profiles",
    ),
    path(
        "payment-profiles/create/",
        views.CustomerPaymentProfileCreateView.as_view(),
        name="create payment profiles",
    ),
    path(
        "payment-profiles/list/",
        views.CustomerPaymentProfileListView.as_view(),
        name="list payment profiles",
    ),
    path(
        "payment-profiles/sync/",
        views.CustomerPaymentProfileSyncView.as_view(),
        name="sync payment profiles",
    ),
    path(
        "payment-profiles/<int:pk>/detail/",
        views.CustomerPaymentProfileDetailView.as_view(),
        name="detail payment profiles",
    ),
    path(
        "payment-profiles/<int:pk>/delete/",
        views.CustomerPaymentProfileDeleteView.as_view(),
        name="delete payment profiles",
    ),
]



from django.test import TestCase


class SubscriptionCreateViewTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
        "terminusgps_payments/tests/test_customeraddressprofile.json",
        "terminusgps_payments/tests/test_customerpaymentprofile.json",
    ]

    def setUp(self):
        self.client.login(
            **{"username": "testuser", "password": "super_secure_password1!"}
        )

    def tearDown(self):
        self.client.logout()

    def test_get_anonymous(self):
        """Fails if a GET request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.get("/subscriptions/create/")
        self.assertEqual(response.status_code, 302)

    def test_get_authenticated(self):
        """Fails if a GET request from an authenticated client returns a response code other than 200."""
        response = self.client.get("/subscriptions/create/")
        self.assertEqual(response.status_code, 200)

    def test_post_anonymous(self):
        """Fails if a POST request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.post("/subscriptions/create/")
        self.assertEqual(response.status_code, 302)

    def test_post_authenticated(self):
        """Fails if a POST request from an authenticated client returns a response code other than 200."""
        response = self.client.post("/subscriptions/create/")
        self.assertEqual(response.status_code, 200)

    def test_not_allowed_http_methods(self):
        """Fails if a non-GET or non-POST request returns a response code other than 405."""
        response = self.client.put("/subscriptions/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.patch("/subscriptions/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.delete("/subscriptions/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.head("/subscriptions/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.options("/subscriptions/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.trace("/subscriptions/create/")
        self.assertEqual(response.status_code, 405)


class CustomerAddressProfileCreateViewTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
    ]

    def setUp(self):
        self.client.login(
            **{"username": "testuser", "password": "super_secure_password1!"}
        )

    def tearDown(self):
        self.client.logout()

    def test_get_anonymous(self):
        """Fails if a GET request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.get("/address-profiles/create/")
        self.assertEqual(response.status_code, 302)

    def test_get_authenticated(self):
        """Fails if a GET request from an authenticated client returns a response code other than 200."""
        response = self.client.get("/address-profiles/create/")
        self.assertEqual(response.status_code, 200)

    def test_post_anonymous(self):
        """Fails if a POST request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.post("/address-profiles/create/")
        self.assertEqual(response.status_code, 302)

    def test_post_authenticated(self):
        """Fails if a POST request from an authenticated client returns a response code other than 200."""
        response = self.client.post("/address-profiles/create/")
        self.assertEqual(response.status_code, 200)

    def test_not_allowed_http_methods(self):
        """Fails if a non-GET or non-POST request returns a response code other than 405."""
        response = self.client.put("/address-profiles/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.patch("/address-profiles/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.delete("/address-profiles/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.head("/address-profiles/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.options("/address-profiles/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.trace("/address-profiles/create/")
        self.assertEqual(response.status_code, 405)


class CustomerAddressProfileDetailViewTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
        "terminusgps_payments/tests/test_customeraddressprofile.json",
    ]

    def setUp(self):
        self.client.login(
            **{"username": "testuser", "password": "super_secure_password1!"}
        )

    def tearDown(self):
        self.client.logout()

    def test_get_anonymous(self):
        """Fails if a GET request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.get("/address-profiles/1/detail/")
        self.assertEqual(response.status_code, 302)

    def test_get_authenticated(self):
        """Fails if a GET request from an authenticated client returns a response code other than 200."""
        response = self.client.get("/address-profiles/1/detail/")
        self.assertEqual(response.status_code, 200)

    def test_not_allowed_http_methods(self):
        """Fails if a non-GET request returns a response code other than 405."""
        response = self.client.post("/address-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.put("/address-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.patch("/address-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.delete("/address-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.head("/address-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.options("/address-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.trace("/address-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)

    def test_get_authenticated_other(self):
        """Fails if a GET request from another authenticated client returns a response code other than 404."""
        self.client.login(
            **{
                "username": "testuseralt",
                "password": "super_secure_password1!",
            }
        )
        response = self.client.get("/address-profiles/1/detail/")
        self.assertEqual(response.status_code, 404)
        self.client.logout()


class CustomerAddressProfileListViewTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
        "terminusgps_payments/tests/test_customeraddressprofile.json",
    ]

    def setUp(self):
        self.client.login(
            **{"username": "testuser", "password": "super_secure_password1!"}
        )

    def tearDown(self):
        self.client.logout()

    def test_get_anonymous(self):
        """Fails if a GET request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.get("/address-profiles/list/")
        self.assertEqual(response.status_code, 302)

    def test_get_authenticated(self):
        """Fails if a GET request from an authenticated client returns a response code other than 200."""
        response = self.client.get("/address-profiles/list/")
        self.assertEqual(response.status_code, 200)
        self.assertInHTML(
            '<a class="hover:underline" href="/address-profiles/1/detail/"><li>TestAddress</li></a>',
            response.rendered_content,
        )
        self.assertNotInHTML(
            '<a class="hover:underline" href="/address-profiles/2/detail/"><li>TestAddress</li></a>',
            response.rendered_content,
        )

    def test_get_authenticated_other(self):
        self.client.login(
            **{
                "username": "testuseralt",
                "password": "super_secure_password1!",
            }
        )
        response = self.client.get("/address-profiles/list/")
        self.assertInHTML(
            '<a class="hover:underline" href="/address-profiles/2/detail/"><li>TestAddress</li></a>',
            response.rendered_content,
        )
        self.assertNotInHTML(
            '<a class="hover:underline" href="/address-profiles/1/detail/"><li>TestAddress</li></a>',
            response.rendered_content,
        )
        self.client.logout()

    def test_not_allowed_http_methods(self):
        """Fails if a non-GET request returns a response code other than 405."""
        response = self.client.post("/address-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.put("/address-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.patch("/address-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.delete("/address-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.head("/address-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.options("/address-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.trace("/address-profiles/list/")
        self.assertEqual(response.status_code, 405)


class CustomerAddressProfileDeleteViewTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
        "terminusgps_payments/tests/test_customeraddressprofile.json",
    ]

    def setUp(self):
        self.client.login(
            **{"username": "testuser", "password": "super_secure_password1!"}
        )

    def tearDown(self):
        self.client.logout()

    def test_get_anonymous(self):
        """Fails if a GET request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.get("/address-profiles/1/delete/")
        self.assertEqual(response.status_code, 302)

    def test_get_authenticated(self):
        """Fails if a GET request from an authenticated client returns a response code other than 200."""
        response = self.client.get("/address-profiles/1/delete/")
        self.assertEqual(response.status_code, 200)

    def test_get_authenticated_other(self):
        """Fails if a GET request from another authenticated client returns a response code other than 404."""
        self.client.login(
            **{
                "username": "testuseralt",
                "password": "super_secure_password1!",
            }
        )
        response = self.client.get("/address-profiles/1/delete/")
        self.assertEqual(response.status_code, 404)
        self.client.logout()

    def test_post_anonymous(self):
        """Fails if a POST request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.post("/address-profiles/1/delete/")
        self.assertEqual(response.status_code, 302)

    def test_post_authenticated(self):
        """Fails if a POST request from an authenticated client returns a response code other than 200."""
        response = self.client.post("/address-profiles/1/delete/")
        self.assertEqual(response.status_code, 200)

    def test_not_allowed_http_methods(self):
        """Fails if a non-GET or non-POST request returns a response code other than 405."""
        response = self.client.put("/address-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)
        response = self.client.patch("/address-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)
        response = self.client.delete("/address-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)
        response = self.client.head("/address-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)
        response = self.client.options("/address-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)
        response = self.client.trace("/address-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)


class CustomerPaymentProfileCreateViewTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
    ]

    def setUp(self):
        self.client.login(
            **{"username": "testuser", "password": "super_secure_password1!"}
        )

    def tearDown(self):
        self.client.logout()

    def test_get_anonymous(self):
        """Fails if a GET request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.get("/address-profiles/create/")
        self.assertEqual(response.status_code, 302)

    def test_get_authenticated(self):
        """Fails if a GET request from an authenticated client returns a response code other than 200."""
        response = self.client.get("/payment-profiles/create/")
        self.assertEqual(response.status_code, 200)

    def test_post_anonymous(self):
        """Fails if a POST request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.post("/payment-profiles/create/")
        self.assertEqual(response.status_code, 302)

    def test_post_authenticated(self):
        """Fails if a POST request from an authenticated client returns a response code other than 200."""
        response = self.client.post("/payment-profiles/create/")
        self.assertEqual(response.status_code, 200)

    def test_not_allowed_http_methods(self):
        """Fails if a non-GET or non-POST request returns a response code other than 405."""
        response = self.client.put("/payment-profiles/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.patch("/payment-profiles/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.delete("/payment-profiles/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.head("/payment-profiles/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.options("/payment-profiles/create/")
        self.assertEqual(response.status_code, 405)
        response = self.client.trace("/payment-profiles/create/")
        self.assertEqual(response.status_code, 405)


class CustomerPaymentProfileDetailViewTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
        "terminusgps_payments/tests/test_customerpaymentprofile.json",
    ]

    def setUp(self):
        self.client.login(
            **{"username": "testuser", "password": "super_secure_password1!"}
        )

    def tearDown(self):
        self.client.logout()

    def test_get_anonymous(self):
        """Fails if a GET request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.get("/address-profiles/1/detail/")
        self.assertEqual(response.status_code, 302)

    def test_get_authenticated(self):
        """Fails if a GET request from an authenticated client returns a response code other than 200."""
        response = self.client.get("/payment-profiles/1/detail/")
        self.assertEqual(response.status_code, 200)

    def test_get_authenticated_other(self):
        """Fails if a GET request from another authenticated client returns a response code other than 404."""
        self.client.login(
            **{
                "username": "testuseralt",
                "password": "super_secure_password1!",
            }
        )
        response = self.client.get("/payment-profiles/1/detail/")
        self.assertEqual(response.status_code, 404)
        self.client.logout()

    def test_not_allowed_http_methods(self):
        """Fails if a non-GET request returns a response code other than 405."""
        response = self.client.post("/payment-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.put("/payment-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.patch("/payment-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.delete("/payment-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.head("/payment-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.options("/payment-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)
        response = self.client.trace("/payment-profiles/1/detail/")
        self.assertEqual(response.status_code, 405)


class CustomerPaymentProfileListViewTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
        "terminusgps_payments/tests/test_customerpaymentprofile.json",
    ]

    def setUp(self):
        self.client.login(
            **{"username": "testuser", "password": "super_secure_password1!"}
        )

    def tearDown(self):
        self.client.logout()

    def test_get_anonymous(self):
        """Fails if a GET request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.get("/payment-profiles/list/")
        self.assertEqual(response.status_code, 302)

    def test_get_authenticated(self):
        """Fails if a GET request from an authenticated client returns a response code other than 200."""
        response = self.client.get("/payment-profiles/list/")
        self.assertEqual(response.status_code, 200)

    def test_get_authenticated_other(self):
        self.client.login(
            **{
                "username": "testuseralt",
                "password": "super_secure_password1!",
            }
        )
        response = self.client.get("/payment-profiles/list/")
        self.assertEqual(response.status_code, 200)
        self.client.logout()

    def test_not_allowed_http_methods(self):
        """Fails if a non-GET request returns a response code other than 405."""
        response = self.client.post("/payment-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.put("/payment-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.patch("/payment-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.delete("/payment-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.head("/payment-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.options("/payment-profiles/list/")
        self.assertEqual(response.status_code, 405)
        response = self.client.trace("/payment-profiles/list/")
        self.assertEqual(response.status_code, 405)


class CustomerPaymentProfileDeleteViewTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
        "terminusgps_payments/tests/test_customerpaymentprofile.json",
    ]

    def setUp(self):
        self.client.login(
            **{"username": "testuser", "password": "super_secure_password1!"}
        )

    def tearDown(self):
        self.client.logout()

    def test_get_anonymous(self):
        """Fails if a GET request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.get("/payment-profiles/1/delete/")
        self.assertEqual(response.status_code, 302)

    def test_get_authenticated(self):
        """Fails if a GET request from an authenticated client returns a response code other than 200."""
        response = self.client.get("/payment-profiles/1/delete/")
        self.assertEqual(response.status_code, 200)

    def test_get_authenticated_other(self):
        """Fails if a GET request from another authenticated client returns a response code other than 404."""
        self.client.login(
            **{
                "username": "testuseralt",
                "password": "super_secure_password1!",
            }
        )
        response = self.client.get("/payment-profiles/1/delete/")
        self.assertEqual(response.status_code, 404)
        self.client.logout()

    def test_post_anonymous(self):
        """Fails if a POST request from an anonymous client returns a response code other than 302."""
        self.client.logout()
        response = self.client.post("/payment-profiles/1/delete/")
        self.assertEqual(response.status_code, 302)

    def test_post_authenticated(self):
        """Fails if a POST request from an authenticated client returns a response code other than 200."""
        response = self.client.post("/payment-profiles/1/delete/")
        self.assertEqual(response.status_code, 200)

    def test_not_allowed_http_methods(self):
        """Fails if a non-GET or non-POST request returns a response code other than 405."""
        response = self.client.put("/payment-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)
        response = self.client.patch("/payment-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)
        response = self.client.delete("/payment-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)
        response = self.client.head("/payment-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)
        response = self.client.options("/payment-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)
        response = self.client.trace("/payment-profiles/1/delete/")
        self.assertEqual(response.status_code, 405)



from django.db.models import QuerySet
from django.views.generic.base import TemplateResponseMixin
from django.views.generic.detail import SingleObjectMixin
from django.views.generic.list import MultipleObjectMixin


class HtmxTemplateResponseMixin(TemplateResponseMixin):
    partial_name: str | None = None

    def render_to_response(self, context, **response_kwargs):
        htmx_request = bool(self.request.headers.get("HX-Request"))
        boosted = bool(self.request.headers.get("HX-Boosted"))
        if htmx_request and not boosted:
            if not self.partial_name:
                self.partial_name = f"{self.template_name}#main"
            self.template_name = self.partial_name
        return super().render_to_response(context, **response_kwargs)


class AuthorizenetSingleObjectMixin(SingleObjectMixin):
    def get_queryset(self) -> QuerySet:
        if hasattr(self.request, "user"):
            return self.model.objects.filter(
                customer_profile__user=self.request.user
            )
        return self.model.objects.none()


class AuthorizenetMultipleObjectMixin(MultipleObjectMixin):
    def get_queryset(self) -> QuerySet:
        if hasattr(self.request, "user"):
            return self.model.objects.filter(
                customer_profile__user=self.request.user
            ).order_by(self.get_ordering())
        return self.model.objects.none()



from django.contrib import admin
from django.urls import include, path

urlpatterns = [
    path("admin/", admin.site.urls),
    path("accounts/", include("django.contrib.auth.urls")),
    path(
        "",
        include("terminusgps_payments.urls", namespace="terminusgps_payments"),
    ),
]



from django.test import RequestFactory, TestCase
from django.views.generic import TemplateView

from terminusgps_payments.views.mixins import HtmxTemplateResponseMixin


class TestTemplateView(HtmxTemplateResponseMixin, TemplateView):
    pass


class HtmxTemplateResponseMixinTestCase(TestCase):
    def setUp(self):
        self.view = TestTemplateView(template_name="test.html")

    def test_render_to_response_explicit_partial_name(self):
        """Fails if :py:attr:`template_name` wasn't set to an explicitly provided :py:attr:`partial_name`."""
        headers = {"HX-Request": "true"}
        request = RequestFactory().get("/", headers=headers)
        explicit_partial_name = "test.html#mypartial"
        self.view.partial_name = explicit_partial_name
        self.view.setup(request)
        self.view.render_to_response(context=self.view.get_context_data())
        self.assertEqual(explicit_partial_name, self.view.template_name)

    def test_render_to_response_htmx_request(self):
        """Fails if :py:attr:`template_name` wasn't updated on an htmx request."""
        headers = {"HX-Request": "true"}
        request = RequestFactory().get("/", headers=headers)
        self.view.setup(request)
        self.view.render_to_response(context=self.view.get_context_data())
        self.assertEqual("test.html#main", self.view.template_name)

    def test_render_to_response_htmx_request_boosted(self):
        """Fails if :py:attr:`template_name` was updated on a boosted htmx request."""
        headers = {"HX-Request": "true", "HX-Boosted": "true"}
        request = RequestFactory().get("/", headers=headers)
        self.view.setup(request)
        self.view.render_to_response(context=self.view.get_context_data())
        self.assertEqual("test.html", self.view.template_name)

    def test_render_to_response_non_htmx_request(self):
        """Fails if :py:attr:`template_name` was updated on a non-htmx request."""
        headers = {}
        request = RequestFactory().get("/", headers=headers)
        self.view.setup(request)
        self.view.render_to_response(context=self.view.get_context_data())
        self.assertEqual("test.html", self.view.template_name)



from django.apps import AppConfig


class TerminusgpsPaymentsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "terminusgps_payments"
    verbose_name = "Terminus GPS Payments"



import typing

from django import forms
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import ValidationError
from django.db import transaction
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.views.generic import (
    CreateView,
    DeleteView,
    DetailView,
    ListView,
    TemplateView,
    UpdateView,
    View,
)
from terminusgps.authorizenet.service import (
    AuthorizenetControllerExecutionError,
    AuthorizenetService,
)

from terminusgps_payments.models import CustomerProfile
from terminusgps_payments.views.mixins import (
    AuthorizenetMultipleObjectMixin,
    AuthorizenetSingleObjectMixin,
    HtmxTemplateResponseMixin,
)


class HtmxTemplateView(HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    http_method_names = ["get"]


class AuthorizenetSyncView(LoginRequiredMixin, View):
    content_type = "text/html"
    http_method_names = ["post"]
    id_attr = None
    list_attr = None
    model = None

    def setup(self, request: HttpRequest, *args, **kwargs) -> None:
        if self.id_attr is None:
            raise ValueError("'id_attr' attribute is required.")
        if self.list_attr is None:
            raise ValueError("'list_attr' attribute is required.")
        if self.model is None:
            raise ValueError("'model' attribute is required.")
        return super().setup(request, *args, **kwargs)

    @transaction.atomic
    def post(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        try:
            customer_profile = CustomerProfile.objects.get(user=request.user)
            service = AuthorizenetService()
            data = customer_profile.pull(service=service)
            if hasattr(data, "profile"):
                qs = self.model.objects.filter(
                    customer_profile=customer_profile
                )
                if not hasattr(data.profile, self.list_attr):
                    qs.delete()
                else:
                    id_list = [
                        int(getattr(subprofile, self.id_attr))
                        for subprofile in getattr(data.profile, self.list_attr)
                    ]
                    for pk in id_list:
                        obj = self.model(
                            pk=pk, customer_profile=customer_profile
                        )
                        obj.save(push=False, service=service)
                    qs.exclude(pk__in=id_list).delete()
            return HttpResponse(status=200)
        except (
            AuthorizenetControllerExecutionError,
            CustomerProfile.DoesNotExist,
        ):
            return HttpResponse(status=400)


class AuthorizenetCreateView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, CreateView
):
    content_type = "text/html"
    form_class = None
    http_method_names = ["get", "post"]
    model = None

    def get_success_url(self):
        return self.success_url

    def form_valid(self, form: forms.ModelForm) -> HttpResponse:
        try:
            self.object = form.save(commit=False)
            self.object.save(push=True)
            return HttpResponseRedirect(self.get_success_url())
        except AuthorizenetControllerExecutionError as error:
            form.add_error(
                None,
                ValidationError(
                    "%(error)s", code="invalid", params={"error": str(error)}
                ),
            )
            return self.form_invalid(form=form)

    def get_initial(self, **kwargs) -> dict[str, typing.Any]:
        initial: dict[str, typing.Any] = super().get_initial(**kwargs)
        try:
            initial["customer_profile"] = (
                CustomerProfile.objects.get(user=self.request.user)
                if hasattr(self.request, "user")
                else None
            )
        except CustomerProfile.DoesNotExist:
            initial["customer_profile"] = None
        return initial


class AuthorizenetListView(
    LoginRequiredMixin,
    HtmxTemplateResponseMixin,
    AuthorizenetMultipleObjectMixin,
    ListView,
):
    allow_empty = True
    content_type = "text/html"
    http_method_names = ["get"]
    ordering = "pk"
    paginate_by = 4


class AuthorizenetDeleteView(
    LoginRequiredMixin,
    HtmxTemplateResponseMixin,
    AuthorizenetSingleObjectMixin,
    DeleteView,
):
    content_type = "text/html"
    http_method_names = ["get", "post"]

    def get_success_url(self):
        return self.success_url

    def form_valid(self, form: forms.ModelForm) -> HttpResponse:
        try:
            service = AuthorizenetService()
            success_url = self.get_success_url()
            self.object._delete_in_authorizenet(service=service)
            self.object.delete()
            return HttpResponseRedirect(success_url)
        except AuthorizenetControllerExecutionError as error:
            form.add_error(
                None,
                ValidationError(
                    "%(error)s", code="invalid", params={"error": str(error)}
                ),
            )
            return self.form_invalid(form=form)


class AuthorizenetDetailView(
    LoginRequiredMixin,
    HtmxTemplateResponseMixin,
    AuthorizenetSingleObjectMixin,
    DetailView,
):
    content_type = "text/html"
    http_method_names = ["get"]


class AuthorizenetUpdateView(
    LoginRequiredMixin,
    HtmxTemplateResponseMixin,
    AuthorizenetSingleObjectMixin,
    UpdateView,
):
    content_type = "text/html"
    http_method_names = ["get"]

    def form_valid(self, form: forms.ModelForm) -> HttpResponse:
        try:
            self.object = form.save(commit=False)
            self.object.save(push=True)
            return HttpResponseRedirect(self.get_success_url())
        except AuthorizenetControllerExecutionError as error:
            form.add_error(
                None,
                ValidationError(
                    "%(error)s", code="invalid", params={"error": str(error)}
                ),
            )
            return self.form_invalid(form=form)



from django.urls import reverse_lazy

from terminusgps_payments.forms import CustomerPaymentProfileCreateForm
from terminusgps_payments.models import CustomerPaymentProfile
from terminusgps_payments.views.generic import (
    AuthorizenetCreateView,
    AuthorizenetDeleteView,
    AuthorizenetDetailView,
    AuthorizenetListView,
    AuthorizenetSyncView,
)


class CustomerPaymentProfileSyncView(AuthorizenetSyncView):
    content_type = "text/html"
    http_method_names = ["get"]
    id_attr = "customerPaymentProfileId"
    list_attr = "paymentProfiles"
    model = CustomerPaymentProfile


class CustomerPaymentProfileCreateView(AuthorizenetCreateView):
    content_type = "text/html"
    form_class = CustomerPaymentProfileCreateForm
    http_method_names = ["get", "post"]
    model = CustomerPaymentProfile
    success_url = reverse_lazy("terminusgps_payments:list payment profiles")
    template_name = "terminusgps_payments/customerpaymentprofile_create.html"


class CustomerPaymentProfileDeleteView(AuthorizenetDeleteView):
    content_type = "text/html"
    http_method_names = ["get", "post"]
    model = CustomerPaymentProfile
    success_url = reverse_lazy("terminusgps_payments:list payment profiles")
    template_name = "terminusgps_payments/customerpaymentprofile_delete.html"


class CustomerPaymentProfileDetailView(AuthorizenetDetailView):
    content_type = "text/html"
    http_method_names = ["get"]
    model = CustomerPaymentProfile
    template_name = "terminusgps_payments/customerpaymentprofile_detail.html"


class CustomerPaymentProfileListView(AuthorizenetListView):
    allow_empty = True
    content_type = "text/html"
    http_method_names = ["get"]
    model = CustomerPaymentProfile
    ordering = "pk"
    paginate_by = 4
    template_name = "terminusgps_payments/customerpaymentprofile_list.html"



from django.contrib import admin

from . import models


@admin.register(models.CustomerProfile)
class CustomerProfileAdmin(admin.ModelAdmin):
    list_display = ["id", "merchant_id", "email"]


@admin.register(models.CustomerPaymentProfile)
class CustomerPaymentProfileAdmin(admin.ModelAdmin):
    list_display = ["id", "customer_profile__merchant_id", "address"]


@admin.register(models.CustomerAddressProfile)
class CustomerAddressProfileAdmin(admin.ModelAdmin):
    list_display = ["id", "customer_profile__merchant_id", "address"]


@admin.register(models.Subscription)
class SubscriptionAdmin(admin.ModelAdmin):
    list_display = [
        "name",
        "amount",
        "status",
        "customer_profile__merchant_id",
    ]
    list_filter = ["status"]



import datetime

from django import forms
from django.core.exceptions import ValidationError
from django.forms import widgets
from django.utils.translation import gettext_lazy as _

from . import models


class ExpirationDateWidget(widgets.MultiWidget):
    def __init__(self, **kwargs) -> None:
        return super().__init__(**kwargs)

    def decompress(self, value):
        if value:
            return [value.month, value.year]
        return [None, None]


class ExpirationDateField(forms.MultiValueField):
    def __init__(self, fields=(), widget=None, **kwargs) -> None:
        return super().__init__(
            error_messages={},
            fields=(
                forms.DateField(input_formats=["%m", "%-m"]),
                forms.DateField(input_formats=["%y"]),
            ),
            widget=widget,
            require_all_fields=False,
            **kwargs,
        )

    def compress(self, data_list):
        if data_list:
            return datetime.date(
                day=1, month=data_list[0].month, year=data_list[1].year
            )
        return None


class CustomerAddressProfileCreateForm(forms.ModelForm):
    class Meta:
        model = models.CustomerAddressProfile
        fields = [
            "first_name",
            "last_name",
            "company",
            "address",
            "city",
            "state",
            "country",
            "zip",
            "phone_number",
            "customer_profile",
        ]
        widgets = {"customer_profile": widgets.HiddenInput()}
        help_texts = {
            "first_name": _("Enter a first name."),
            "last_name": _("Enter a last name."),
            "company": _("Optional. Enter a company name."),
            "address": _("Enter a house number + street."),
            "city": _("Enter a city."),
            "state": _("Enter a state."),
            "zip": _("Enter a ZIP code."),
            "country": _("Enter a country."),
            "phone_number": _(
                "Optional. Enter an E.164-formatted phone number. Ex: 17139045262"
            ),
        }

    def clean(self):
        cleaned_data = super().clean()
        shipping_address_fields = [
            cleaned_data.get("first_name"),
            cleaned_data.get("last_name"),
            cleaned_data.get("address"),
            cleaned_data.get("city"),
            cleaned_data.get("state"),
            cleaned_data.get("country"),
            cleaned_data.get("zip"),
        ]
        if not all(shipping_address_fields):
            raise ValidationError(
                _("Please fill out all required shipping address fields."),
                code="invalid",
            )


class CustomerPaymentProfileCreateForm(forms.ModelForm):
    class Meta:
        model = models.CustomerPaymentProfile
        field_classes = {"card_expiry": ExpirationDateField}
        fields = [
            "first_name",
            "last_name",
            "company",
            "address",
            "city",
            "state",
            "country",
            "zip",
            "phone_number",
            "card_number",
            "card_expiry",
            "card_code",
            "account_number",
            "routing_number",
            "account_name",
            "account_type",
            "bank_name",
            "customer_profile",
        ]
        help_texts = {
            "first_name": _("Enter a first name."),
            "last_name": _("Enter a last name."),
            "company": _("Optional. Enter a company name."),
            "address": _("Enter a house number + street."),
            "city": _("Enter a city."),
            "state": _("Enter a state."),
            "zip": _("Enter a ZIP code."),
            "country": _("Enter a country."),
            "phone_number": _(
                "Optional. Enter an E.164-formatted phone number. Ex: 17139045262"
            ),
            "card_number": _("Enter a card number."),
            "card_expiry": _("Enter an expiration date month and year."),
            "card_code": _("Enter a 3-4 digit CCV code."),
            "account_number": _("Enter an account number."),
            "routing_number": _("Enter a routing number."),
            "account_name": _("Enter an account holder name."),
            "bank_name": _("Enter a bank name."),
            "account_type": _("Select a bank account type."),
        }
        widgets = {
            "expiry_date": ExpirationDateWidget(
                widgets={
                    "month": widgets.DateInput(format=["%m"]),
                    "year": widgets.DateInput(format=["%y"]),
                }
            ),
            "customer_profile": widgets.HiddenInput(),
        }

    def clean(self) -> None:
        cleaned_data = super().clean()
        billing_address_fields = [
            cleaned_data.get("first_name"),
            cleaned_data.get("last_name"),
            cleaned_data.get("address"),
            cleaned_data.get("city"),
            cleaned_data.get("state"),
            cleaned_data.get("country"),
            cleaned_data.get("zip"),
        ]
        credit_card_fields = [
            cleaned_data.get("card_number"),
            cleaned_data.get("card_expiry"),
            cleaned_data.get("card_code"),
        ]
        bank_account_fields = [
            cleaned_data.get("account_number"),
            cleaned_data.get("routing_number"),
            cleaned_data.get("account_name"),
            cleaned_data.get("account_type"),
            cleaned_data.get("bank_name"),
        ]

        if not any(billing_address_fields):
            raise ValidationError(
                _("Please fill out all required billing address fields."),
                code="invalid",
            )
        if any(credit_card_fields) and any(bank_account_fields):
            raise ValidationError(
                _(
                    "Please fill out all credit card fields or bank account fields, not both."
                ),
                code="invalid",
            )
        if any(credit_card_fields) and not all(credit_card_fields):
            raise ValidationError(
                _("Please fill out all credit card fields."), code="invalid"
            )
        if any(bank_account_fields) and not all(bank_account_fields):
            raise ValidationError(
                _("Please fill out all bank account fields."), code="invalid"
            )



# Generated by Django 6.0.1 on 2026-02-02 19:52

import datetime
from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('terminusgps_payments', '0002_customerprofile_user'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscription',
            name='amount',
            field=models.DecimalField(decimal_places=2, default=Decimal('24.95'), max_digits=15),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='start_date',
            field=models.DateField(blank=True, default=datetime.date.today),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='trial_amount',
            field=models.DecimalField(decimal_places=2, default=Decimal('0.00'), max_digits=15),
        ),
    ]



# Generated by Django 6.0.1 on 2026-01-26 18:29

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('terminusgps_payments', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='customerprofile',
            name='user',
            field=models.OneToOneField(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='customer_profile', to=settings.AUTH_USER_MODEL),
            preserve_default=False,
        ),
    ]



from datetime import date

from django.test import TestCase

from terminusgps_payments.forms import (
    CustomerAddressProfileCreateForm,
    CustomerPaymentProfileCreateForm,
)
from terminusgps_payments.models import CustomerProfile


class CustomerAddressProfileCreateFormTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
    ]

    def setUp(self):
        self.form_cls = CustomerAddressProfileCreateForm
        self.address_data = {
            "first_name": "TestFirstName",
            "last_name": "TestLastName",
            "company": "TestCompany",
            "address": "TestAddress",
            "city": "TestCity",
            "state": "TestState",
            "country": "TestCountry",
            "zip": "TestZip",
            "phone_number": "TestPhoneNumber",
        }

    def test_clean_missing_address(self):
        """Fails if the form doesn't have the proper error message applied with missing address data."""
        data = {"customer_profile": CustomerProfile.objects.get(pk=1)}
        form = self.form_cls(data=data)
        self.assertFalse(form.is_valid())
        self.assertFormError(
            form, None, "Please fill out all required shipping address fields."
        )


class CustomerPaymentProfileCreateFormTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
    ]

    def setUp(self):
        self.form_cls = CustomerPaymentProfileCreateForm
        self.card_data = {
            "card_number": "4111111111111111",
            "card_expiry": [date(2039, 12, 1), date(2039, 12, 1)],
            "card_code": "444",
        }
        self.address_data = {
            "first_name": "TestFirstName",
            "last_name": "TestLastName",
            "company": "TestCompany",
            "address": "TestAddress",
            "city": "TestCity",
            "state": "TestState",
            "country": "TestCountry",
            "zip": "TestZip",
            "phone_number": "TestPhoneNumber",
        }
        self.bank_account_data = {
            "account_number": "41111111111111111",
            "routing_number": "411111111",
            "account_name": "TestAccountName",
            "account_type": "checking",
            "bank_name": "TestBankName",
        }

    def test_clean_missing_address(self):
        """Fails if the form doesn't have the proper error message applied with missing address data."""
        data = {"customer_profile": CustomerProfile.objects.get(pk=1)}
        form = self.form_cls(data=data)
        self.assertFalse(form.is_valid())
        self.assertFormError(
            form, None, "Please fill out all required billing address fields."
        )

    def test_clean_both_credit_card_and_bank_account(self):
        """Fails if the form doesn't have the proper error message applied with both credit card and bank account data."""
        data = (
            {"customer_profile": CustomerProfile.objects.get(pk=1)}
            | self.address_data.copy()
            | self.card_data.copy()
            | self.bank_account_data.copy()
        )
        form = self.form_cls(data=data)
        self.assertFalse(form.is_valid())
        self.assertFormError(
            form,
            None,
            "Please fill out all credit card fields or bank account fields, not both.",
        )

    def test_clean_credit_card(self):
        """Fails if the form doesn't have the proper error message applied with invalid credit card data."""
        data = (
            {"customer_profile": CustomerProfile.objects.get(pk=1)}
            | self.address_data.copy()
            | self.card_data.copy()
        )
        data["card_number"] = ""
        form = self.form_cls(data=data)
        self.assertFalse(form.is_valid())
        self.assertFormError(
            form, None, "Please fill out all credit card fields."
        )

    def test_clean_bank_account(self):
        """Fails if the form doesn't have the proper error message applied with invalid bank account data."""
        data = (
            {"customer_profile": CustomerProfile.objects.get(pk=1)}
            | self.address_data.copy()
            | self.bank_account_data.copy()
        )
        data["account_number"] = ""
        form = self.form_cls(data=data)
        self.assertFalse(form.is_valid())
        self.assertFormError(
            form, None, "Please fill out all bank account fields."
        )



# Generated by Django 6.0.1 on 2026-01-21 20:41

import django.db.models.deletion
from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CustomerProfile',
            fields=[
                ('id', models.PositiveBigIntegerField(blank=True, primary_key=True, serialize=False)),
                ('email', models.EmailField(blank=True, max_length=255)),
                ('merchant_id', models.CharField(blank=True, max_length=20)),
                ('description', models.TextField(blank=True, max_length=255)),
            ],
            options={
                'verbose_name': 'customer profile',
                'verbose_name_plural': 'customer profiles',
            },
        ),
        migrations.CreateModel(
            name='CustomerPaymentProfile',
            fields=[
                ('id', models.PositiveBigIntegerField(blank=True, primary_key=True, serialize=False)),
                ('is_default', models.BooleanField(default=False)),
                ('first_name', models.CharField(blank=True, max_length=50)),
                ('last_name', models.CharField(blank=True, max_length=50)),
                ('company', models.CharField(blank=True, max_length=50)),
                ('address', models.CharField(blank=True, max_length=60)),
                ('city', models.CharField(blank=True, max_length=40)),
                ('state', models.CharField(blank=True, max_length=40)),
                ('zip', models.CharField(blank=True, max_length=20)),
                ('country', models.CharField(blank=True, max_length=60)),
                ('phone_number', models.CharField(blank=True, max_length=25)),
                ('card_number', models.CharField(blank=True, max_length=16)),
                ('card_expiry', models.DateField(blank=True, default=None, null=True)),
                ('card_code', models.CharField(blank=True, max_length=4)),
                ('card_type', models.CharField(blank=True)),
                ('account_type', models.CharField(blank=True, choices=[('checking', 'Checking'), ('savings', 'Savings'), ('businessChecking', 'Business checking')])),
                ('account_number', models.CharField(blank=True, max_length=17)),
                ('routing_number', models.CharField(blank=True, max_length=9)),
                ('account_name', models.CharField(blank=True, max_length=22)),
                ('echeck_type', models.CharField(blank=True, choices=[('PPD', 'PPD'), ('WEB', 'WEB'), ('CCD', 'CCD')])),
                ('bank_name', models.CharField(blank=True, max_length=50)),
                ('customer_profile', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='payment_profiles', to='terminusgps_payments.customerprofile')),
            ],
            options={
                'verbose_name': 'payment profile',
                'verbose_name_plural': 'payment profiles',
            },
        ),
        migrations.CreateModel(
            name='CustomerAddressProfile',
            fields=[
                ('id', models.PositiveBigIntegerField(blank=True, primary_key=True, serialize=False)),
                ('is_default', models.BooleanField(default=False)),
                ('first_name', models.CharField(blank=True, max_length=50)),
                ('last_name', models.CharField(blank=True, max_length=50)),
                ('company', models.CharField(blank=True, max_length=50)),
                ('address', models.CharField(blank=True, max_length=60)),
                ('city', models.CharField(blank=True, max_length=40)),
                ('state', models.CharField(blank=True, max_length=40)),
                ('zip', models.CharField(blank=True, max_length=20)),
                ('country', models.CharField(blank=True, max_length=60)),
                ('phone_number', models.CharField(blank=True, max_length=25)),
                ('customer_profile', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='address_profiles', to='terminusgps_payments.customerprofile')),
            ],
            options={
                'verbose_name': 'address profile',
                'verbose_name_plural': 'address profiles',
            },
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.PositiveBigIntegerField(blank=True, primary_key=True, serialize=False)),
                ('name', models.CharField(blank=True, max_length=50)),
                ('interval_length', models.IntegerField(default=1)),
                ('interval_unit', models.CharField(choices=[('days', 'Days'), ('months', 'Months')], default='months')),
                ('start_date', models.DateField(blank=True, default=None, null=True)),
                ('total_occurrences', models.IntegerField(default=9999)),
                ('trial_occurrences', models.IntegerField(default=0)),
                ('amount', models.DecimalField(blank=True, decimal_places=2, max_digits=15)),
                ('trial_amount', models.DecimalField(blank=True, decimal_places=2, default=Decimal('0.00'), max_digits=15)),
                ('status', models.CharField(choices=[('active', 'Active'), ('expired', 'Expired'), ('suspended', 'Suspended'), ('canceled', 'Canceled'), ('terminated', 'Terminated')], default='active')),
                ('address_profile', models.ForeignKey(on_delete=django.db.models.deletion.RESTRICT, related_name='subscriptions', to='terminusgps_payments.customeraddressprofile')),
                ('customer_profile', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='subscriptions', to='terminusgps_payments.customerprofile')),
                ('payment_profile', models.ForeignKey(on_delete=django.db.models.deletion.RESTRICT, related_name='subscriptions', to='terminusgps_payments.customerpaymentprofile')),
            ],
            options={
                'verbose_name': 'subscription',
                'verbose_name_plural': 'subscriptions',
            },
        ),
    ]



import typing

from django import forms
from django.core.exceptions import ValidationError
from django.http import HttpResponse, HttpResponseRedirect
from django.urls import reverse_lazy
from terminusgps.authorizenet.service import (
    AuthorizenetControllerExecutionError,
    AuthorizenetService,
)

from terminusgps_payments.models import (
    CustomerAddressProfile,
    CustomerPaymentProfile,
    CustomerProfile,
    Subscription,
)
from terminusgps_payments.views.generic import (
    AuthorizenetCreateView,
    AuthorizenetDeleteView,
    AuthorizenetDetailView,
    AuthorizenetUpdateView,
)


class SubscriptionCreateView(AuthorizenetCreateView):
    content_type = "text/html"
    fields = [
        "name",
        "interval_length",
        "interval_unit",
        "start_date",
        "total_occurrences",
        "trial_occurrences",
        "amount",
        "trial_amount",
        "customer_profile",
        "payment_profile",
        "address_profile",
    ]
    http_method_names = ["get", "post"]
    model = Subscription
    template_name = "terminusgps_payments/subscription_create.html"

    def form_valid(self, form: forms.ModelForm) -> HttpResponse:
        try:
            self.object = form.save(commit=False)
            self.object.save(push=True)
            return HttpResponseRedirect(self.object.get_absolute_url())
        except AuthorizenetControllerExecutionError as error:
            match error.code:
                case _:
                    form.add_error(
                        None,
                        ValidationError(
                            "%(error)s",
                            code="invalid",
                            params={"error": error},
                        ),
                    )
            return self.form_invalid(form=form)

    def get_initial(self, **kwargs) -> dict[str, typing.Any]:
        try:
            customer_profile = CustomerProfile.objects.get(
                user=self.request.user
            )
        except CustomerProfile.DoesNotExist:
            customer_profile = None

        initial: dict[str, typing.Any] = super().get_initial(**kwargs)
        initial["customer_profile"] = customer_profile
        return initial

    def get_form(self, form_class=None) -> forms.Form:
        payment_qs = (
            CustomerPaymentProfile.objects.filter(
                customer_profile__user=self.request.user
            )
            if hasattr(self.request, "user")
            else CustomerPaymentProfile.objects.none()
        )
        address_qs = (
            CustomerAddressProfile.objects.filter(
                customer_profile__user=self.request.user
            )
            if hasattr(self.request, "user")
            else CustomerAddressProfile.objects.none()
        )

        form = super().get_form(form_class=form_class)
        form.fields["payment_profile"].queryset = payment_qs
        form.fields["address_profile"].queryset = address_qs
        form.fields["payment_profile"].empty_label = None
        form.fields["address_profile"].empty_label = None
        return form


class SubscriptionDetailView(AuthorizenetDetailView):
    content_type = "text/html"
    http_method_names = ["get"]
    model = Subscription
    template_name = "terminusgps_payments/subscription_detail.html"


class SubscriptionDeleteView(AuthorizenetDeleteView):
    content_type = "text/html"
    http_method_names = ["get", "post"]
    model = Subscription
    success_url = reverse_lazy("terminusgps_payments:create subscriptions")
    template_name = "terminusgps_payments/subscription_delete.html"

    def form_valid(self, form: forms.ModelForm) -> HttpResponse:
        try:
            service = AuthorizenetService()
            self.object._delete_in_authorizenet(service=service)
            self.object.delete()
            return HttpResponseRedirect(self.success_url)
        except AuthorizenetControllerExecutionError as error:
            match error.code:
                case _:
                    form.add_error(
                        None,
                        ValidationError(
                            "%(error)s",
                            code="invalid",
                            params={"error": error},
                        ),
                    )
            return self.form_invalid(form=form)


class SubscriptionUpdateView(AuthorizenetUpdateView):
    content_type = "text/html"
    fields = ["address_profile", "payment_profile"]
    http_method_names = ["get", "post"]
    model = Subscription
    template_name = "terminusgps_payments/subscription_update.html"

    def get_success_url(self):
        return self.object.get_absolute_url()

    def get_form(self, form_class=None) -> forms.Form:
        payment_qs = (
            CustomerPaymentProfile.objects.filter(
                customer_profile__user=self.request.user
            )
            if hasattr(self.request, "user")
            else CustomerPaymentProfile.objects.none()
        )
        address_qs = (
            CustomerAddressProfile.objects.filter(
                customer_profile__user=self.request.user
            )
            if hasattr(self.request, "user")
            else CustomerAddressProfile.objects.none()
        )

        form = super().get_form(form_class=form_class)
        form.fields["payment_profile"].queryset = payment_qs
        form.fields["address_profile"].queryset = address_qs
        form.fields["payment_profile"].empty_label = None
        form.fields["address_profile"].empty_label = None
        return form



from django.urls import reverse_lazy

from terminusgps_payments.forms import CustomerAddressProfileCreateForm
from terminusgps_payments.models import CustomerAddressProfile
from terminusgps_payments.views.generic import (
    AuthorizenetCreateView,
    AuthorizenetDeleteView,
    AuthorizenetDetailView,
    AuthorizenetListView,
    AuthorizenetSyncView,
)


class CustomerAddressProfileSyncView(AuthorizenetSyncView):
    content_type = "text/html"
    http_method_names = ["get"]
    id_attr = "customerAddressId"
    list_attr = "shipToList"
    model = CustomerAddressProfile


class CustomerAddressProfileCreateView(AuthorizenetCreateView):
    content_type = "text/html"
    form_class = CustomerAddressProfileCreateForm
    http_method_names = ["get", "post"]
    model = CustomerAddressProfile
    success_url = reverse_lazy("terminusgps_payments:list address profiles")
    template_name = "terminusgps_payments/customeraddressprofile_create.html"


class CustomerAddressProfileDeleteView(AuthorizenetDeleteView):
    content_type = "text/html"
    http_method_names = ["get", "post"]
    model = CustomerAddressProfile
    success_url = reverse_lazy("terminusgps_payments:list address profiles")
    template_name = "terminusgps_payments/customeraddressprofile_delete.html"


class CustomerAddressProfileDetailView(AuthorizenetDetailView):
    content_type = "text/html"
    http_method_names = ["get"]
    model = CustomerAddressProfile
    template_name = "terminusgps_payments/customeraddressprofile_detail.html"


class CustomerAddressProfileListView(AuthorizenetListView):
    allow_empty = True
    content_type = "text/html"
    http_method_names = ["get"]
    model = CustomerAddressProfile
    ordering = "pk"
    paginate_by = 4
    template_name = "terminusgps_payments/customeraddressprofile_list.html"



import abc
import datetime
import decimal
import logging

from authorizenet import apicontractsv1
from django.contrib.auth import get_user_model
from django.db import models
from django.urls import reverse
from django.utils.dateparse import parse_date
from django.utils.translation import gettext_lazy as _
from lxml.objectify import ObjectifiedElement
from terminusgps.authorizenet import api
from terminusgps.authorizenet.service import AuthorizenetService

logger = logging.getLogger(__name__)


class AuthorizenetModel(models.Model):
    id = models.PositiveBigIntegerField(primary_key=True, blank=True)
    """Authorizenet object id."""

    class Meta:
        abstract = True

    def save(self, **kwargs) -> None:
        service = kwargs.pop("service", AuthorizenetService())
        ref = kwargs.pop("reference_id", None)
        if not kwargs.pop("push", False) and self.pk:
            logger.debug(f"Syncing #{self.pk} with Authorizenet...")
            elem = self.pull(service, reference_id=ref)
            self.sync(elem)
            logger.debug(f"Synced #{self.pk} with Authorizenet.")
        else:
            resp = self.push(service, reference_id=ref)
            if not self.pk:
                self.pk = self._extract_authorizenet_id(resp)
            logger.debug(f"Pushed #{self.pk} to Authorizenet.")
        return super().save(**kwargs)

    @abc.abstractmethod
    def push(
        self,
        service: AuthorizenetService,
        reference_id: str | None = None,
        **kwargs,
    ) -> ObjectifiedElement:
        raise NotImplementedError("Subclasses must implement this method.")

    @abc.abstractmethod
    def pull(
        self,
        service: AuthorizenetService,
        reference_id: str | None = None,
        **kwargs,
    ) -> ObjectifiedElement:
        raise NotImplementedError("Subclasses must implement this method.")

    @abc.abstractmethod
    def sync(self, elem: ObjectifiedElement, **kwargs) -> None:
        raise NotImplementedError("Subclasses must implement this method.")

    @abc.abstractmethod
    def _extract_authorizenet_id(self, elem: ObjectifiedElement) -> int:
        raise NotImplementedError("Subclasses must implement this method.")

    @abc.abstractmethod
    def _delete_in_authorizenet(
        self, service: AuthorizenetService, reference_id: str | None = None
    ) -> None:
        raise NotImplementedError("Subclasses must implement this method.")


class CustomerProfile(AuthorizenetModel):
    user = models.OneToOneField(
        get_user_model(),
        related_name="customer_profile",
        on_delete=models.CASCADE,
    )
    email = models.EmailField(blank=True, max_length=255)
    merchant_id = models.CharField(blank=True, max_length=20)
    description = models.TextField(blank=True, max_length=255)

    class Meta:
        verbose_name = _("customer profile")
        verbose_name_plural = _("customer profiles")

    def __str__(self) -> str:
        return (
            str(self.merchant_id)
            if self.merchant_id
            else f"CustomerProfile #{self.pk}"
        )

    def push(
        self, service: AuthorizenetService, reference_id: str | None = None
    ) -> ObjectifiedElement:
        if not self.pk:
            request = api.create_customer_profile(
                merchant_id=str(self.merchant_id),
                email=str(self.email),
                description=str(self.description),
            )
        else:
            request = api.update_customer_profile(
                profile=apicontractsv1.customerProfileExType(
                    customerProfileId=str(self.pk),
                    merchantCustomerId=str(self.merchant_id),
                    description=str(self.description),
                    email=str(self.email),
                )
            )
        return service.execute(request, reference_id=reference_id)

    def pull(
        self,
        service: AuthorizenetService,
        reference_id: str | None = None,
        include_issuer_info: bool = False,
    ) -> ObjectifiedElement:
        request = api.get_customer_profile(
            customer_profile_id=self.pk,
            include_issuer_info=include_issuer_info,
        )
        return service.execute(request, reference_id=reference_id)

    def sync(self, elem: ObjectifiedElement) -> None:
        if hasattr(elem, "profile"):
            self.merchant_id = getattr(elem.profile, "merchantCustomerId", "")
            self.email = getattr(elem.profile, "email", "")
            self.description = getattr(elem.profile, "description", "")
        return

    def _delete_in_authorizenet(
        self, service: AuthorizenetService, reference_id: str | None = None
    ) -> None:
        request = api.delete_customer_profile(customer_profile_id=self.pk)
        service.execute(request, reference_id=reference_id)
        return

    def _extract_authorizenet_id(self, elem: ObjectifiedElement) -> int:
        return int(getattr(elem, "customerProfileId"))


class CustomerAddressProfile(AuthorizenetModel):
    customer_profile = models.ForeignKey(
        "terminusgps_payments.CustomerProfile",
        on_delete=models.CASCADE,
        related_name="address_profiles",
    )

    is_default = models.BooleanField(default=False)
    first_name = models.CharField(blank=True, max_length=50)
    last_name = models.CharField(blank=True, max_length=50)
    company = models.CharField(blank=True, max_length=50)
    address = models.CharField(blank=True, max_length=60)
    city = models.CharField(blank=True, max_length=40)
    state = models.CharField(blank=True, max_length=40)
    zip = models.CharField(blank=True, max_length=20)
    country = models.CharField(blank=True, max_length=60)
    phone_number = models.CharField(blank=True, max_length=25)

    class Meta:
        verbose_name = _("address profile")
        verbose_name_plural = _("address profiles")

    def __str__(self) -> str:
        if self.address:
            return str(self.address)
        return f"CustomerAddressProfile #{self.pk}"

    def get_absolute_url(self) -> str:
        return reverse(
            "terminusgps_payments:detail address profiles",
            kwargs={"pk": self.pk},
        )

    def push(
        self, service: AuthorizenetService, reference_id: str | None = None
    ) -> ObjectifiedElement:
        address = apicontractsv1.customerAddressType()
        address.firstName = str(self.first_name)
        address.lastName = str(self.last_name)
        address.company = str(self.company)
        address.address = str(self.address)
        address.city = str(self.city)
        address.state = str(self.state)
        address.zip = str(self.zip)
        address.country = str(self.country)
        address.phoneNumber = str(self.phone_number)

        if not self.pk:
            request = api.create_customer_shipping_address(
                customer_profile_id=self.customer_profile.pk,
                address=address,
                default=self.is_default,
            )
        else:
            request = api.update_customer_shipping_address(
                customer_profile_id=self.customer_profile.pk,
                address_profile_id=self.pk,
                address=address,
                default=self.is_default,
            )
        return service.execute(request, reference_id=reference_id)

    def pull(
        self, service: AuthorizenetService, reference_id: str | None = None
    ) -> ObjectifiedElement:
        request = api.get_customer_shipping_address(
            customer_profile_id=self.customer_profile.pk,
            address_profile_id=self.pk,
        )
        return service.execute(request, reference_id=reference_id)

    def sync(self, elem: ObjectifiedElement) -> None:
        if hasattr(elem, "defaultShippingAddress"):
            self.is_default = bool(getattr(elem, "defaultShippingAddress"))
        if hasattr(elem, "address"):
            self.first_name = getattr(elem.address, "firstName", "")
            self.last_name = getattr(elem.address, "lastName", "")
            self.company = getattr(elem.address, "company", "")
            self.address = getattr(elem.address, "address", "")
            self.city = getattr(elem.address, "city", "")
            self.state = getattr(elem.address, "state", "")
            self.country = getattr(elem.address, "country", "")
            self.zip = getattr(elem.address, "zip", "")
            self.phone_number = getattr(elem.address, "phoneNumber", "")
        return

    def _extract_authorizenet_id(self, elem: ObjectifiedElement) -> int:
        return int(getattr(elem, "customerAddressId"))

    def _delete_in_authorizenet(
        self, service: AuthorizenetService, reference_id: str | None = None
    ) -> None:
        request = api.delete_customer_shipping_address(
            customer_profile_id=self.customer_profile.pk,
            address_profile_id=self.pk,
        )
        service.execute(request, reference_id=reference_id)


class CustomerPaymentProfile(AuthorizenetModel):
    class ECheckType(models.TextChoices):
        PPD = "PPD", _("PPD")
        WEB = "WEB", _("WEB")
        CCD = "CCD", _("CCD")

    class AccountType(models.TextChoices):
        CHECKING = "checking", _("Checking")
        SAVINGS = "savings", _("Savings")
        BUSINESS_CHECKING = "businessChecking", _("Business checking")

    customer_profile = models.ForeignKey(
        "terminusgps_payments.CustomerProfile",
        on_delete=models.CASCADE,
        related_name="payment_profiles",
    )
    is_default = models.BooleanField(default=False)

    first_name = models.CharField(blank=True, max_length=50)
    last_name = models.CharField(blank=True, max_length=50)
    company = models.CharField(blank=True, max_length=50)
    address = models.CharField(blank=True, max_length=60)
    city = models.CharField(blank=True, max_length=40)
    state = models.CharField(blank=True, max_length=40)
    zip = models.CharField(blank=True, max_length=20)
    country = models.CharField(blank=True, max_length=60)
    phone_number = models.CharField(blank=True, max_length=25)

    card_number = models.CharField(blank=True, max_length=16)
    card_expiry = models.DateField(blank=True, default=None, null=True)
    card_code = models.CharField(blank=True, max_length=4)
    card_type = models.CharField(blank=True)

    account_type = models.CharField(blank=True, choices=AccountType.choices)
    account_number = models.CharField(blank=True, max_length=17)
    routing_number = models.CharField(blank=True, max_length=9)
    account_name = models.CharField(blank=True, max_length=22)
    echeck_type = models.CharField(blank=True, choices=ECheckType.choices)
    bank_name = models.CharField(blank=True, max_length=50)

    class Meta:
        verbose_name = _("payment profile")
        verbose_name_plural = _("payment profiles")

    def __str__(self) -> str:
        if self.card_number and self.card_type:
            return f"{self.card_type} {self.card_number}"
        elif self.account_type and self.bank_name:
            return f"{self.bank_name} {self.account_type}"
        else:
            return f"CustomerPaymentProfile #{self.pk}"

    def save(self, **kwargs) -> None:
        super().save(**kwargs)
        if self.pk and self.card_number:
            self.card_code = ""
            if not str(self.card_number).startswith("XXXX"):
                last_4 = str(self.card_number)[-4:]
                self.card_number = f"XXXX{last_4}"

    def get_absolute_url(self) -> str:
        return reverse(
            "terminusgps_payments:detail payment profiles",
            kwargs={"pk": self.pk},
        )

    def push(
        self, service: AuthorizenetService, reference_id: str | None = None
    ) -> ObjectifiedElement:
        payment = apicontractsv1.paymentType()
        if self.card_number:
            payment.creditCard = apicontractsv1.creditCardType()
            payment.creditCard.cardNumber = str(self.card_number)
            if self.card_expiry is not None:
                payment.creditCard.expirationDate = self.card_expiry.strftime(
                    format="%Y-%m"
                )
            if self.card_code:
                payment.creditCard.cardCode = str(self.card_code)
        if all(
            [
                self.account_number,
                self.routing_number,
                self.account_name,
                self.bank_name,
                self.account_type,
                self.echeck_type,
            ]
        ):
            payment.bankAccount = apicontractsv1.bankAccountType()
            payment.bankAccount.accountType = str(self.account_type)
            payment.bankAccount.routingNumber = str(self.routing_number)
            payment.bankAccount.accountNumber = str(self.account_number)
            payment.bankAccount.nameOnAccount = str(self.account_name)
            payment.bankAccount.echeckType = str(self.echeck_type)
            payment.bankAccount.bankName = str(self.bank_name)

        address = apicontractsv1.customerAddressType()
        address.firstName = str(self.first_name)
        address.lastName = str(self.last_name)
        address.company = str(self.company)
        address.address = str(self.address)
        address.city = str(self.city)
        address.state = str(self.state)
        address.zip = str(self.zip)
        address.country = str(self.country)
        address.phoneNumber = str(self.phone_number)

        if not self.pk:
            request = api.create_customer_payment_profile(
                customer_profile_id=self.customer_profile.pk,
                payment=payment,
                address=address,
                default=self.is_default,
            )
        else:
            request = api.update_customer_payment_profile(
                customer_profile_id=self.customer_profile.pk,
                payment_profile_id=self.pk,
                payment=payment,
                address=address,
                default=self.is_default,
            )
        service.execute(request, reference_id=reference_id)

    def pull(
        self,
        service: AuthorizenetService,
        reference_id: str | None = None,
        include_issuer_info: bool = False,
    ) -> ObjectifiedElement:
        request = api.get_customer_payment_profile(
            customer_profile_id=self.customer_profile.pk,
            payment_profile_id=self.pk,
            include_issuer_info=include_issuer_info,
        )
        return service.execute(request, reference_id=reference_id)

    def sync(self, elem: ObjectifiedElement) -> None:
        if hasattr(elem, "defaultPaymentProfile"):
            self.is_default = bool(getattr(elem, "defaultPaymentProfile"))
        if hasattr(elem, "paymentProfile"):
            if hasattr(elem.paymentProfile, "billTo"):
                address = elem.paymentProfile.billTo
                self.first_name = getattr(address, "firstName", "")
                self.last_name = getattr(address, "lastName", "")
                self.company = getattr(address, "company", "")
                self.address = getattr(address, "address", "")
                self.city = getattr(address, "city", "")
                self.state = getattr(address, "state", "")
                self.country = getattr(address, "country", "")
                self.zip = getattr(address, "zip", "")
                self.phone_number = getattr(address, "phoneNumber", "")
            if hasattr(elem.paymentProfile, "payment"):
                payment = elem.paymentProfile.payment
                if hasattr(payment, "creditCard"):
                    ca = payment.creditCard
                    self.card_number = str(getattr(ca, "cardNumber", ""))
                    self.card_type = str(getattr(ca, "cardType", ""))
                if hasattr(payment, "bankAccount"):
                    ba = payment.bankAccount
                    self.account_type = str(getattr(ba, "accountType", ""))
                    self.account_number = str(getattr(ba, "accountNumber", ""))
                    self.routing_number = str(getattr(ba, "routingNumber", ""))
                    self.account_name = str(getattr(ba, "nameOnAccount", ""))
                    self.echeck_type = str(getattr(ba, "eCheckType", ""))
                    self.bank_name = str(getattr(ba, "bankName", ""))
        return

    def _extract_authorizenet_id(self, elem: ObjectifiedElement) -> int:
        return int(getattr(elem, "customerPaymentProfileId"))

    def _delete_in_authorizenet(
        self, service: AuthorizenetService, reference_id: str | None = None
    ) -> None:
        request = api.delete_customer_payment_profile(
            customer_profile_id=self.customer_profile.pk,
            payment_profile_id=self.pk,
        )
        return service.execute(request, reference_id=reference_id)


class Subscription(AuthorizenetModel):
    class SubscriptionStatus(models.TextChoices):
        ACTIVE = "active", _("Active")
        EXPIRED = "expired", _("Expired")
        SUSPENDED = "suspended", _("Suspended")
        CANCELED = "canceled", _("Canceled")
        TERMINATED = "terminated", _("Terminated")

    class SubscriptionIntervalUnit(models.TextChoices):
        DAYS = apicontractsv1.ARBSubscriptionUnitEnum.days, _("Days")
        MONTHS = apicontractsv1.ARBSubscriptionUnitEnum.months, _("Months")

    name = models.CharField(blank=True, max_length=50)
    interval_length = models.IntegerField(default=1)
    interval_unit = models.CharField(
        choices=SubscriptionIntervalUnit.choices,
        default=SubscriptionIntervalUnit.MONTHS,
    )
    start_date = models.DateField(blank=True, default=datetime.date.today)
    total_occurrences = models.IntegerField(default=9999)
    trial_occurrences = models.IntegerField(default=0)
    amount = models.DecimalField(
        decimal_places=2, default=decimal.Decimal("24.95"), max_digits=15
    )
    trial_amount = models.DecimalField(
        decimal_places=2, default=decimal.Decimal("0.00"), max_digits=15
    )
    status = models.CharField(
        choices=SubscriptionStatus.choices, default=SubscriptionStatus.ACTIVE
    )

    customer_profile = models.ForeignKey(
        "terminusgps_payments.CustomerProfile",
        on_delete=models.CASCADE,
        related_name="subscriptions",
    )
    payment_profile = models.ForeignKey(
        "terminusgps_payments.CustomerPaymentProfile",
        on_delete=models.RESTRICT,
        related_name="subscriptions",
    )
    address_profile = models.ForeignKey(
        "terminusgps_payments.CustomerAddressProfile",
        on_delete=models.RESTRICT,
        related_name="subscriptions",
    )

    class Meta:
        verbose_name = _("subscription")
        verbose_name_plural = _("subscriptions")

    def __str__(self) -> str:
        return str(self.name) if self.name else f"Subscription #{self.pk}"

    def get_absolute_url(self):
        return reverse(
            "terminusgps_payments:detail subscriptions", kwargs={"pk": self.pk}
        )

    def push(
        self, service: AuthorizenetService, reference_id: str | None = None
    ) -> ObjectifiedElement:
        schedule = apicontractsv1.paymentScheduleType()
        schedule.startDate = self.start_date.strftime("%Y-%m-%d")
        schedule.totalOccurrences = str(self.total_occurrences)
        schedule.trialOccurrences = str(self.trial_occurrences)

        profile = apicontractsv1.customerProfileIdType()
        profile.customerProfileId = str(self.customer_profile.pk)
        profile.customerPaymentProfileId = str(self.payment_profile.pk)
        profile.customerAddressId = str(self.address_profile.pk)

        subscription = apicontractsv1.ARBSubscriptionType()
        subscription.name = str(self.name)
        subscription.paymentSchedule = schedule
        subscription.amount = str(self.amount)
        subscription.trialAmount = str(self.trial_amount)
        subscription.profile = profile

        if not self.pk:
            interval = apicontractsv1.paymentScheduleTypeInterval()
            interval.unit = str(self.interval_unit)
            interval.length = str(self.interval_length)
            subscription.paymentSchedule.interval = interval
            request = api.create_subscription(subscription=subscription)
        else:
            request = api.update_subscription(
                subscription_id=self.pk, subscription=subscription
            )
        return service.execute(request, reference_id=reference_id)

    def pull(
        self,
        service: AuthorizenetService,
        reference_id: str | None = None,
        include_transactions: bool = True,
    ) -> ObjectifiedElement:
        request = api.get_subscription(
            subscription_id=self.pk, include_transactions=include_transactions
        )
        return service.execute(request, reference_id=reference_id)

    def sync(self, elem: ObjectifiedElement) -> None:
        if hasattr(elem, "subscription"):
            self.name = getattr(elem.subscription, "name", "")
            self.amount = getattr(elem.subscription, "amount", "")
            self.trial_amount = getattr(elem.subscription, "trialAmount", "")
            self.status = getattr(elem.subscription, "status", "active")
            if hasattr(elem.subscription, "paymentSchedule"):
                sch = elem.subscription.paymentSchedule
                self.total_occurrences = int(
                    getattr(sch, "totalOccurrences", 0)
                )
                self.trial_occurrences = int(
                    getattr(sch, "trialOccurrences", 0)
                )
                start_date = str(getattr(sch, "startDate", ""))
                if start_date:
                    self.start_date = parse_date(start_date)
        return

    def _extract_authorizenet_id(self, elem: ObjectifiedElement) -> int:
        return int(elem.subscriptionId)

    def _delete_in_authorizenet(
        self, service: AuthorizenetService, reference_id: str | None = None
    ) -> None:
        request = api.cancel_subscription(subscription_id=self.pk)
        return service.execute(request, reference_id=reference_id)



import logging
from datetime import date
from unittest.mock import Mock

from authorizenet import apicontrollers
from django.test import TestCase, override_settings
from lxml import objectify
from terminusgps.authorizenet.service import AuthorizenetService

from terminusgps_payments.models import (
    CustomerAddressProfile,
    CustomerPaymentProfile,
    CustomerProfile,
    Subscription,
)

logging.disable(logging.CRITICAL)


class CustomerProfileTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
    ]

    def setUp(self):
        self.customerprofile = CustomerProfile.objects.get(pk=1)

    def test___str__(self):
        """Fails if the customer profile's :py:meth:`__str__` method returned unexpected values."""
        self.assertEqual(
            str(self.customerprofile), str(self.customerprofile.merchant_id)
        )
        self.customerprofile.merchant_id = ""
        self.assertIn(str(self.customerprofile.pk), str(self.customerprofile))

    def test_push_create(self):
        """Fails if :py:meth:`push` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.createCustomerProfileController`."""
        mock_service = Mock(AuthorizenetService)

        self.customerprofile.pk = None
        self.customerprofile.push(mock_service)

        expected = apicontrollers.createCustomerProfileController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_push_update(self):
        """Fails if :py:meth:`push` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.updateCustomerProfileController`."""
        mock_service = Mock(AuthorizenetService)

        self.customerprofile.pk = 1
        self.customerprofile.push(mock_service)

        expected = apicontrollers.updateCustomerProfileController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_pull(self):
        """Fails if :py:meth:`pull` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.getCustomerProfileController`."""
        mock_service = Mock(AuthorizenetService)

        self.customerprofile.pk = 1
        self.customerprofile.pull(mock_service)

        expected = apicontrollers.getCustomerProfileController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_sync(self):
        """Fails if :py:attr:`email`, :py:attr:`merchant_id` or :py:attr:`description` weren't synced on :py:meth:`sync`."""
        merchant_id = "merchantId"
        email = "email@domain.com"
        description = "description"

        self.customerprofile.sync(
            elem=objectify.E.root(
                objectify.E.profile(
                    objectify.E.merchantCustomerId(merchant_id),
                    objectify.E.email(email),
                    objectify.E.description(description),
                )
            )
        )

        self.assertEqual(self.customerprofile.email, email)
        self.assertEqual(self.customerprofile.merchant_id, merchant_id)
        self.assertEqual(self.customerprofile.description, description)

    def test__delete_in_authorizenet(self):
        """Fails if :py:meth:`_delete_in_authorizenet` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.deleteCustomerProfileController`."""
        mock_service = Mock(AuthorizenetService)

        self.customerprofile.pk = 1
        self.customerprofile._delete_in_authorizenet(mock_service)

        expected = apicontrollers.deleteCustomerProfileController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test__extract_authorizenet_id(self):
        """Fails if :py:meth:`_extract_authorizenet_id` returns a non-integer."""
        expected = 1
        mock_element = objectify.E.root(
            objectify.E.customerProfileId(str(expected))
        )

        result = self.customerprofile._extract_authorizenet_id(mock_element)
        self.assertIsInstance(result, int)
        self.assertEqual(result, 1)

    def test_save_pull_and_sync(self):
        """"""
        mock_service = Mock(AuthorizenetService)
        mock_service.execute.return_value = objectify.E.root(
            objectify.E.profile(
                objectify.E.merchantCustomerId("TestMerchantId1"),
                objectify.E.email("test1@domain.com"),
                objectify.E.description("TestDescription1"),
            )
        )

        self.customerprofile.save(service=mock_service)

    def test_save_push(self):
        """"""
        mock_service = Mock(AuthorizenetService)
        mock_service.execute.return_value = objectify.E.root(
            objectify.E.customerProfileId(1)
        )

        self.customerprofile.pk = None
        self.customerprofile.save(service=mock_service)


@override_settings(ROOT_URLCONF="src.urls")
class CustomerAddressProfileTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
        "terminusgps_payments/tests/test_customeraddressprofile.json",
    ]

    def setUp(self):
        self.customeraddressprofile = CustomerAddressProfile.objects.get(pk=1)

    def test___str__(self):
        """Fails if the address profile's :py:meth:`__str__` method returned unexpected values."""
        self.assertEqual(
            str(self.customeraddressprofile),
            str(self.customeraddressprofile.address),
        )
        self.customeraddressprofile.pk = 1
        self.customeraddressprofile.address = ""
        self.assertEqual(
            str(self.customeraddressprofile), "CustomerAddressProfile #1"
        )

    def test_get_absolute_url(self):
        url = self.customeraddressprofile.get_absolute_url()
        self.assertEqual("/address-profiles/1/detail/", url)

    def test_push_create(self):
        """Fails if :py:meth:`push` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.createCustomerShippingAddressController`."""
        mock_service = Mock(AuthorizenetService)

        self.customeraddressprofile.pk = None
        self.customeraddressprofile.push(mock_service)

        expected = apicontrollers.createCustomerShippingAddressController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_push_update(self):
        """Fails if :py:meth:`push` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.updateCustomerShippingAddressController`."""
        mock_service = Mock(AuthorizenetService)

        self.customeraddressprofile.pk = 1
        self.customeraddressprofile.push(mock_service)

        expected = apicontrollers.updateCustomerShippingAddressController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_pull(self):
        """Fails if :py:meth:`pull` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.getCustomerShippingAddressController`."""
        mock_service = Mock(AuthorizenetService)

        self.customeraddressprofile.pk = 1
        self.customeraddressprofile.pull(mock_service)

        expected = apicontrollers.getCustomerShippingAddressController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_sync(self):
        """Fails if address profile attributes weren't synced on :py:meth:`sync`."""
        first_name = "firstName"
        last_name = "lastName"
        company = "company"
        address = "address"
        city = "city"
        state = "state"
        country = "country"
        zip = "zip"
        phone_number = "phoneNumber"

        self.customeraddressprofile.sync(
            elem=objectify.E.root(
                objectify.E.defaultShippingAddress(1),
                objectify.E.address(
                    objectify.E.firstName(first_name),
                    objectify.E.lastName(last_name),
                    objectify.E.company(company),
                    objectify.E.address(address),
                    objectify.E.city(city),
                    objectify.E.state(state),
                    objectify.E.country(country),
                    objectify.E.zip(zip),
                    objectify.E.phoneNumber(phone_number),
                ),
            )
        )

        self.assertEqual(self.customeraddressprofile.first_name, first_name)
        self.assertEqual(self.customeraddressprofile.last_name, last_name)
        self.assertEqual(self.customeraddressprofile.company, company)
        self.assertEqual(self.customeraddressprofile.address, address)
        self.assertEqual(self.customeraddressprofile.city, city)
        self.assertEqual(self.customeraddressprofile.state, state)
        self.assertEqual(self.customeraddressprofile.country, country)
        self.assertEqual(self.customeraddressprofile.zip, zip)
        self.assertEqual(
            self.customeraddressprofile.phone_number, phone_number
        )

    def test__delete_in_authorizenet(self):
        """Fails if :py:meth:`_delete_in_authorizenet` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.deleteCustomerShippingAddressController`."""
        mock_service = Mock(AuthorizenetService)

        self.customeraddressprofile.pk = 1
        self.customeraddressprofile._delete_in_authorizenet(mock_service)

        expected = apicontrollers.deleteCustomerShippingAddressController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test__extract_authorizenet_id(self):
        """Fails if :py:meth:`_extract_authorizenet_id` returns a non-integer."""
        expected = 1
        mock_element = objectify.E.root(
            objectify.E.customerAddressId(str(expected))
        )

        result = self.customeraddressprofile._extract_authorizenet_id(
            mock_element
        )
        self.assertIsInstance(result, int)
        self.assertEqual(result, 1)


@override_settings(ROOT_URLCONF="src.urls")
class CustomerPaymentProfileTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
        "terminusgps_payments/tests/test_customerpaymentprofile.json",
    ]

    def test___str__(self):
        """Fails if the payment profile's :py:meth:`__str__` method returned unexpected values."""
        cc_customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=1)
        ba_customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=2)
        self.assertEqual(
            str(cc_customerpaymentprofile), "TestCardType XXXX1111"
        )
        self.assertEqual(
            str(ba_customerpaymentprofile), "TestBankName checking"
        )
        cc_customerpaymentprofile.card_number = ""
        cc_customerpaymentprofile.card_type = ""
        self.assertEqual(
            str(cc_customerpaymentprofile), "CustomerPaymentProfile #1"
        )

    def test_get_absolute_url(self):
        customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=1)
        url = customerpaymentprofile.get_absolute_url()
        self.assertEqual("/payment-profiles/1/detail/", url)

    def test_push_create_credit_card(self):
        """Fails if :py:meth:`push` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.createCustomerPaymentProfileController` for a credit card."""
        mock_service = Mock(AuthorizenetService)
        customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=1)

        customerpaymentprofile.pk = None
        customerpaymentprofile.push(mock_service)

        expected = apicontrollers.createCustomerPaymentProfileController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_push_create_bank_account(self):
        """Fails if :py:meth:`push` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.createCustomerPaymentProfileController` for a bank account."""
        mock_service = Mock(AuthorizenetService)
        customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=2)

        customerpaymentprofile.pk = None
        customerpaymentprofile.push(mock_service)

        expected = apicontrollers.createCustomerPaymentProfileController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_push_update(self):
        """Fails if :py:meth:`push` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.updateCustomerPaymentProfileController`."""
        mock_service = Mock(AuthorizenetService)
        customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=1)

        customerpaymentprofile.push(mock_service)

        expected = apicontrollers.updateCustomerPaymentProfileController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_pull(self):
        """Fails if :py:meth:`pull` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.getCustomerPaymentProfileController`."""
        mock_service = Mock(AuthorizenetService)
        customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=1)

        customerpaymentprofile.pull(mock_service)

        expected = apicontrollers.getCustomerPaymentProfileController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_sync_bank_account(self):
        """Fails if payment profile attributes weren't synced on :py:meth:`sync` with a bank account."""
        first_name = "firstName"
        last_name = "lastName"
        company = "company"
        address = "address"
        city = "city"
        state = "state"
        country = "country"
        zip = "zip"
        phone_number = "phoneNumber"
        account_type = "accountType"
        account_number = "accountNumber"
        routing_number = "routingNumber"
        account_name = "nameOnAccount"
        echeck_type = "eCheckType"
        bank_name = "bankName"

        customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=2)
        customerpaymentprofile.sync(
            elem=objectify.E.root(
                objectify.E.defaultPaymentProfile(1),
                objectify.E.billTo(
                    objectify.E.firstName(first_name),
                    objectify.E.lastName(last_name),
                    objectify.E.company(company),
                    objectify.E.address(address),
                    objectify.E.city(city),
                    objectify.E.state(state),
                    objectify.E.country(country),
                    objectify.E.zip(zip),
                    objectify.E.phoneNumber(phone_number),
                ),
                objectify.E.paymentProfile(
                    objectify.E.payment(
                        objectify.E.bankAccount(
                            objectify.E.accountType(account_type),
                            objectify.E.accountNumber(account_number),
                            objectify.E.routingNumber(routing_number),
                            objectify.E.nameOnAccount(account_name),
                            objectify.E.eCheckType(echeck_type),
                            objectify.E.bankName(bank_name),
                        )
                    )
                ),
            )
        )

        self.assertEqual(customerpaymentprofile.first_name, first_name)
        self.assertEqual(customerpaymentprofile.last_name, last_name)
        self.assertEqual(customerpaymentprofile.company, company)
        self.assertEqual(customerpaymentprofile.address, address)
        self.assertEqual(customerpaymentprofile.city, city)
        self.assertEqual(customerpaymentprofile.state, state)
        self.assertEqual(customerpaymentprofile.country, country)
        self.assertEqual(customerpaymentprofile.zip, zip)
        self.assertEqual(customerpaymentprofile.phone_number, phone_number)
        self.assertEqual(customerpaymentprofile.account_type, account_type)
        self.assertEqual(customerpaymentprofile.account_number, account_number)
        self.assertEqual(customerpaymentprofile.routing_number, routing_number)
        self.assertEqual(customerpaymentprofile.account_name, account_name)
        self.assertEqual(customerpaymentprofile.echeck_type, echeck_type)
        self.assertEqual(customerpaymentprofile.bank_name, bank_name)

    def test_sync_credit_card(self):
        """Fails if payment profile attributes weren't synced on :py:meth:`sync` with a credit card."""
        first_name = "firstName"
        last_name = "lastName"
        company = "company"
        address = "address"
        city = "city"
        state = "state"
        country = "country"
        zip = "zip"
        phone_number = "phoneNumber"
        card_number = "cardNumber"
        card_type = "cardType"

        customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=1)
        customerpaymentprofile.sync(
            elem=objectify.E.root(
                objectify.E.defaultPaymentProfile(1),
                objectify.E.billTo(
                    objectify.E.firstName(first_name),
                    objectify.E.lastName(last_name),
                    objectify.E.company(company),
                    objectify.E.address(address),
                    objectify.E.city(city),
                    objectify.E.state(state),
                    objectify.E.country(country),
                    objectify.E.zip(zip),
                    objectify.E.phoneNumber(phone_number),
                ),
                objectify.E.paymentProfile(
                    objectify.E.payment(
                        objectify.E.creditCard(
                            objectify.E.cardNumber(card_number),
                            objectify.E.cardType(card_type),
                        )
                    )
                ),
            )
        )

        self.assertEqual(customerpaymentprofile.first_name, first_name)
        self.assertEqual(customerpaymentprofile.last_name, last_name)
        self.assertEqual(customerpaymentprofile.company, company)
        self.assertEqual(customerpaymentprofile.address, address)
        self.assertEqual(customerpaymentprofile.city, city)
        self.assertEqual(customerpaymentprofile.state, state)
        self.assertEqual(customerpaymentprofile.country, country)
        self.assertEqual(customerpaymentprofile.zip, zip)
        self.assertEqual(customerpaymentprofile.phone_number, phone_number)
        self.assertEqual(customerpaymentprofile.card_number, card_number)
        self.assertEqual(customerpaymentprofile.card_type, card_type)

    def test__delete_in_authorizenet(self):
        """Fails if :py:meth:`_delete_in_authorizenet` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.deleteCustomerPaymentProfileController`."""
        mock_service = Mock(AuthorizenetService)
        customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=1)

        customerpaymentprofile._delete_in_authorizenet(mock_service)

        expected = apicontrollers.deleteCustomerPaymentProfileController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test__extract_authorizenet_id(self):
        """Fails if :py:meth:`_extract_authorizenet_id` returns a non-integer."""
        customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=1)
        expected_pk = 123
        mock_element = objectify.E.root(
            objectify.E.customerPaymentProfileId(str(expected_pk))
        )

        result = customerpaymentprofile._extract_authorizenet_id(mock_element)
        self.assertIsInstance(result, int)
        self.assertEqual(result, expected_pk)

    def test_save_obfuscates_card_number(self):
        """Fails if calling :py:meth:`save` didn't obfuscate the payment profile credit card."""
        customerpaymentprofile = CustomerPaymentProfile.objects.get(pk=1)
        customerpaymentprofile.card_number = "4111111111111111"
        customerpaymentprofile.card_code = "444"
        customerpaymentprofile.save(
            service=Mock(AuthorizenetService), push=True
        )
        self.assertEqual(customerpaymentprofile.card_number, "XXXX1111")
        self.assertEqual(customerpaymentprofile.card_code, "")


class SubscriptionTestCase(TestCase):
    fixtures = [
        "terminusgps_payments/tests/test_user.json",
        "terminusgps_payments/tests/test_customerprofile.json",
        "terminusgps_payments/tests/test_customerpaymentprofile.json",
        "terminusgps_payments/tests/test_customeraddressprofile.json",
        "terminusgps_payments/tests/test_subscription.json",
    ]

    def test_exclusive_address_profiles(self):
        return

    def test_exclusive_payment_profiles(self):
        return

    def test_pull(self):
        """Fails if :py:meth:`pull` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.getCustomerProfileController`."""
        mock_service = Mock(AuthorizenetService)
        subscription = Subscription.objects.get(pk=1)
        subscription.pull(mock_service)

        expected = apicontrollers.ARBGetSubscriptionController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_push_create(self):
        """Fails if :py:meth:`push` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.createCustomerProfileController`."""
        mock_service = Mock(AuthorizenetService)
        subscription = Subscription.objects.get(pk=1)
        subscription.pk = None

        subscription.push(mock_service)

        expected = apicontrollers.ARBCreateSubscriptionController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_push_update(self):
        """Fails if :py:meth:`push` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.updateCustomerProfileController`."""
        mock_service = Mock(AuthorizenetService)
        subscription = Subscription.objects.get(pk=1)

        subscription.push(mock_service)

        expected = apicontrollers.ARBUpdateSubscriptionController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test_sync(self):
        """Fails if subscription attributes weren't synced on :py:meth:`sync`."""
        subscription = Subscription.objects.get(pk=1)
        name = "TestName"
        amount = "TestAmount"
        trial_amount = "TestTrialAmount"
        status = "TestStatus"
        total_occurrences = 9999
        trial_occurrences = 0
        start_date = date(2039, 12, 1)

        subscription.sync(
            elem=objectify.E.root(
                objectify.E.subscription(
                    objectify.E.name(name),
                    objectify.E.amount(amount),
                    objectify.E.trialAmount(trial_amount),
                    objectify.E.status(status),
                    objectify.E.paymentSchedule(
                        objectify.E.totalOccurrences(total_occurrences),
                        objectify.E.trialOccurrences(trial_occurrences),
                        objectify.E.startDate(start_date.strftime("%Y-%m-%d")),
                    ),
                )
            )
        )

        self.assertEqual(subscription.name, name)
        self.assertEqual(subscription.amount, amount)
        self.assertEqual(subscription.trial_amount, trial_amount)
        self.assertEqual(subscription.status, status)
        self.assertEqual(subscription.total_occurrences, total_occurrences)
        self.assertEqual(subscription.trial_occurrences, trial_occurrences)
        self.assertEqual(subscription.start_date, start_date)

    def test__delete_in_authorizenet(self):
        """Fails if :py:meth:`_delete_in_authorizenet` used an Authorizenet API contoller other than :py:obj:`~authorizenet.apicontrollers.deleteCustomerShippingAddressController`."""
        mock_service = Mock(AuthorizenetService)
        subscription = Subscription.objects.get(pk=1)

        subscription._delete_in_authorizenet(mock_service)

        expected = apicontrollers.ARBCancelSubscriptionController
        controller = mock_service.execute.call_args.args[0][1]
        self.assertEqual(controller, expected)

    def test__extract_authorizenet_id(self):
        """Fails if :py:meth:`_extract_authorizenet_id` returns a non-integer."""
        subscription = Subscription.objects.get(pk=1)
        expected = 1
        mock_element = objectify.E.root(
            objectify.E.subscriptionId(str(expected))
        )

        result = subscription._extract_authorizenet_id(mock_element)

        self.assertIsInstance(result, int)
        self.assertEqual(result, 1)
